#!/usr/bin/php -q
<?php

define('SKIP_KOHANA', true);
require('/opt/monitor/op5/ninja/index.php');

$base_path = dirname(__FILE__).'/../';
require_once($base_path.'/include/config.inc.php');

$examples="
Examples:
CREATE NEW HOST:
	php monitor.php -t host -o host_name=API_TESTHOST -o alias=API_TESTHOST -o address=192.168.1.26 -o template=default-host-template -o contacts=monitor -u monitor
ADD SERVICE:
	php monitor.php -t service -o template=default-service -o host_name=API_TESTHOST -o service_description=API_TEST_SERVICE -o check_command=check_ping  -o check_command_args=100,20%\!500,60% -u monitor
ADD CONTACT:
	php monitor.php -t contact -o contact_name=API_CONTACT  -o alias=API_CONTACT -o email=per@op5.com -o service_notification_period=none -o host_notification_period=none -u monitor
ADD TIMEPERIOD
	php monitor.php -t timeperiod -o timeperiod_name=API_TIMEPERIOD -o alias=API_TIMEPERIOD -u monitor
DELETE SERVICE:
	php monitor.php -t service -n \"API_TESTHOST;API_TESTSERVICE\" -a delete -u monitor
DELETE HOST:
	php monitor.php -t host -n API_TESTHOST -a delete -u monitor
DELETE CONTACT:
	php monitor.php -t contact -n API_CONTACT -a delete -u monitor
DELETE HOST:
	php monitor.php -t host -n API_TESTHOST -a delete -u monitor
DELETE TIMEPERIOD
	php monitor.php -t timeperiod -a delete -n API_TIMEPERIOD -u monitor
SHOW HOST
	php monitor.php -t host -a show_object -n API_TESTHOST -u monitor
SHOW SERVICE
	php monitor.php -t service -a show_object -n \"API_TESTHOST;API_TESTSERVICE\" -u monitor
LIST AVAILABLE OBJECTS OF TYPE
	php monitor.php -t service_template -a list_objects -u monitor
SAVE CONFIGURATION TO FILE:
	php monitor.php -a save_config -u monitor
UNDO CONFIGURATION CHANGES (READ CONFIG FROM FILE);
	php monitor.php -a undo_config -u monitor
SHOW PENDING CHANGES:
	php monitor.php -a show_changes

Note 1: that the '!' char need to be escaped with a backslash '\' as in the service example above.
Note 2: also that the service object differ from all other object since it is not only identified by
the service description, hostname is also needed. A service is identified by the
hostname;service_description tuple, separated by a semicolon.
Note 3: All commands need to be acompanied with -u <user> as in the examples above. If the default
user 'monitor' has been removed or his privileges reduced a different user need to be used.
";

$workflow="
Workflow:
When working with the Nacoma API it is important to know a few things about op5 Monitor operations.
First of all, op5 Monitor uses configuration files, the Nacoma database is only used to create the
configuration files. When saving the configuration using the save button in the webinterface or the
'-a save_config' commandline option the database is written to file and the op5 Monitor process is
re-loaded. The reverse function is the 'Undo submitted changes' button in the webinterface and the
'-a undo_config' commandline option. The fetches the configuration from file and populates the
database.

When using the commandline API you normally start by issuing '-a undo_config'. This is good
practice since we otherwise might write changes made in the webinterface to file that was never
meant to be saved to file. When all changes are submitted to database we issue '-a save_config' to
write the configuration to file. IE to add 2 hosts with one service and delete a contact we do:
1, php monitor.php -a undo_config -u monitor
2, php monitor.php -t host -o host_name=API_TESTHOST_1 -o alias=API_TESTHOST_1 -o address=192.168.1.26 -o template=default-host-template -o contacts=monitor -u monitor
3, php monitor.php -t service -o template=default-service -o host_name=API_TESTHOST_1 -o service_description=API_TEST_SERVICE -o check_command=check_ping -o check_command_args=100,20%\!500,60% -u monitor
4, php monitor.php -t host -o host_name=API_TESTHOST_2 -o alias=API_TESTHOST_2 -o address=192.168.1.27 -o template=default-host-template -o contacts=monitor -u monitor
5, php monitor.php -t service -o template=default-service -o host_name=API_TESTHOST_2 -o service_description=API_TEST_SERVICE -o check_command=check_ping -o check_command_args=100,20%\!500,60% -u monitor
6, php monitor.php -t contact -n API_CONTACT -a delete -u monitor
7, php monitor.php -a save_config -u monitor

";

$shortopts  = "";
$shortopts .= "t:"; // object type - required
$shortopts .= "n:"; // (old)name
$shortopts .= "R:"; // Re-name
$shortopts .= "i:"; // ID
$shortopts .= "u:"; // user, affects the user field in changelog
$shortopts .= "a:"; // action
$shortopts .= "o:"; // options (many calls possible)
$shortopts .= "H::"; // Show help
$shortopts .= "E::"; // Show help, examples and workflow

/**
 * Print usage information.
 * @param $msg [optional] Printed if supplied
 */
function usage($msg)
{
	print("This is the commandline API towards the Nacoma database.\n");
	print("Usage:\n");
	print(basename($_SERVER["argv"][0]) ." -t|--obj_type <object type>\n");
	print(" -n <string> \t (Old) name of object\n");
	print(" -R <string> \t rename object (parameter=new_name)\n");
	print(" -i <ID>\t Object ID in database\n");
	print(" -o <array>\t Additional options for object \n");
	print(" -a <string>\t [delete, save_config, undo_config, show_changes] \n");
	print(" -H     \t Show this help \n");
	print(" -E     \t Show this help + examples \n");

	if (!empty($msg)) {
		print("\n*!* ".$msg."\n");
	}
	exit(1);
}

$class_name			= false;
$opts 				= getopt($shortopts);
$obj 				= false;
$obj_type 			= false;
$obj_id 			= false;
$obj_name 			= false;
$new_field_name 	= false;
$new_field_value 	= false;
$action 			= false;
$options 			= false;
$erray 				= false;
$success			= false;
$console_access		= true;
$user_name			= false;

if(empty($opts)){
	usage($examples);
}

give_infinite_rights();

foreach ($opts as $key => $arg)
{
	switch ($key) {
		case 'R': case 'r': // rename object
			$tmp_parts = false;
			if (strstr($arg, '=')) {
				$tmp_parts = explode('=', $arg);
				if (!empty($tmp_parts)) {
					$new_field_name 	= $tmp_parts[0];
					$new_field_value 	= $tmp_parts[1];
				}
			}
			break;
		case 'o': // Options (array)
			$tmp_parts = false;
			if (is_array($arg)) {
				foreach ($arg as $k) {
					if (strstr($k, '=')) {
						$tmp_parts = explode('=', $k);
						if (!empty($tmp_parts)) {
							$options[$tmp_parts[0]] = $tmp_parts[1];
						}
					}
				}
			} else {
				if (strstr($arg, '=')) {
					$tmp_parts = explode('=', $arg);
					if (!empty($tmp_parts)) {
						$options[$tmp_parts[0]] = $tmp_parts[1];
					}
				}
			}
			break;
		case 'u':
			$user_name = $arg;
			give_infinite_rights($user_name);
			break;
		case 'H':
			usage('');
			break;
		case 'E':
#			usage($workflow);
			usage($examples . $workflow);
			break;
		case 'a': case 't': case 'i': case 'n':
			//fallthrough
			break;
		default:
			usage("Unrecognized option '$key'");
	}
}

if ($owner = nacoma_objlock::get_lock_owner('all', 0)) {
	print "The database is currently being exported by $owner\n".
		"This should merely take a few seconds, but while it's going on, no other users may edit the database.\n";
	exit(0);
}

require_once('api.inc.php');

if (isset($opts['a'])) {
	$action = $opts['a'];
	unset($opts['a']);
}

if (isset($opts['t'])) {
	$obj_type = addslashes(trim(strtolower($opts['t'])));
	$class_name = $obj_type . '_nacoma_Model';
	if (!class_exists($class_name) || !is_subclass_of($class_name, 'object_nacoma_Model')) {
		usage("Unrecognized object type ('$obj_type')");
	}
	unset($opts['t']);
}
else {
	switch ($action) {
	 case 'save_config':
	 case 'undo_config':
	 case 'show_changes':
		break;
	 default:
		usage("Object type required");
	}
}

// if an import is required, we need to make sure
// that it happens before we try to fetch any objects from the database
if (!get_last_import_time() || changes_since_last_import()) {
	// Possibly someone will want to parse the output, so don't print anything
	import_config(!empty($user_name) ? $user_name : 'console user');
}

if (isset($opts['i'])) {
	$obj_id = (int)$opts['i'];
	$obj = new $class_name($obj_id);
	unset($opts['i']);
}
else if (isset($opts['n'])) {
	$obj_name = $opts['n'];
	$obj = new $class_name();
	$obj->get_object_by_name($obj_name);
	unset($opts['n']);
}
else if ($class_name) {
	$obj = new $class_name();
}

switch ($action) {
	case 'delete':
		if (is_object($obj) && !empty($obj->obj)) {
			// prolly ok
			if (!$obj->banish_object()) {
				echo $obj->get_deletion_message();
				exit(0);
			}
			$erray = true;
		} else {
			echo "No saved object named '$obj_name' of type '$obj_type' was found.\n";
			exit(0);
		}
		break;
	case 'list_objects':
		$obj_list = api_list_objects($obj_type);
		if (!empty($obj)) {
			echo $obj_list."\n";
			$erray = true;
		} else {
			$erray = false;
		}
		break;
	case 'show_object':
		$obj_list = api_show_object($obj);
		if (!empty($obj_list)) {
			echo $obj_list;
			$erray = true;
		} else {
			echo "Unable to find the requested object '$obj_name'\n";
			$erray = false;
		}
		break;
	case 'save_config':
		$access = get_nacoma_access('export');
		if (!in_array(NACOMA_ACCESS_FULL_ACCESS, $access)) {
			$erray = array('export' => "You're not authorized to save config changes");
			break;
		}
		# write db to file
		chdir($base_path);
		$_SERVER['REMOTE_USER'] = !empty($user_name) ? $user_name : 'console user'; # ugly but it works
		include_once($base_path.'include/export.inc.php');
		// Ugh, lousy crap interface
		//
		// If we were provided with an ID to export, only export that
		if ($obj_id)
			$changes = new ChangeLog_nacoma_Model(null, $obj_id);
		// If we were provided with an object name to export, only export that
		else if ($obj_name)
			$changes = new ChangeLog_nacoma_Model(null, $obj_name);
		// If we were provided with a username, only export that user's changes
		else if ($user_name)
			$changes = new ChangeLog_nacoma_Model();
		// Else, rewrite the whole shebang
		else
			$changes = true;

		if (($err = do_export($changes)) !== true) {
			$erray = array('export' => $err['type']);
		}
		else {
			switch(export_reload_monitor()) {
			 case 1:
				$erray = array('restart' => "Monitor doesn't appear to be running.\nThis could indicate a more serious error, so restarting not attempted.");
				break;
			 case 2:
				$erray = array('restart' => "Failed to restart monitor!\n");
				break;
			 default:
				$erray = true;
				break;
			}
		}
		break;
	case 'undo_config':
		$access = get_nacoma_access('export');
		if (!in_array(NACOMA_ACCESS_FULL_ACCESS, $access)) {
			$erray = array('export' => "You're not authorized to save config changes");
			break;
		}
		# re-read config from file
		$user_name = !empty($user_name) ? $user_name : 'console user';
		$erray = import_config($user_name);
		break;
	case 'show_changes':
		$access = get_nacoma_access('export');
		if (!in_array(NACOMA_ACCESS_FULL_ACCESS, $access)) {
			$erray = array('export' => "You're not authorized to save config changes");
			break;
		}
		$changes = new ChangeLog_nacoma_Model(null, null, $user_name == false);
		print $changes->get_text_list();
		$erray=true;
		break;
	default: // edit
		$erray = edit_object($obj, $options, $new_field_name, $new_field_value);
		break;
}

unset($obj);
$success = api_display_error($erray);
if ($success) {
	exit(0);
}
exit(1);
