<?php
$apc = function_exists('apc_fetch'); // is the APC php module installed?

$authmodule_fields = array(
    'common' =>  array(
        'session_key' => array('input' => 'string', 'value' => ''),
        'default_auth' => array('input' => 'select', 'value' => ''), //Select between drivers
        'enable_auto_login' => array('input' => 'bool', 'value' => '1'), // Auto Login for apache.
        'apc_enabled' => array('input' => 'bool', 'value' => '1', 'apc' => $apc), //true, false. Cannot be true if apc is not installed.
        'apc_ttl' => array('input' => 'int', 'value' => '', 'apc' => $apc), //greyed out if apc is not installed
        'apc_store_prefix' => array('input' => 'string', 'value' => '', 'apc' => $apc) //greyed out if apc is not installed
    ),
    'Default' => array(
        'rename_module' => array('input' => 'string', 'value' => '')
    ),
    'LDAP' => array(
        'server' => array('input' => 'string', 'value' => ''),
        'port' => array('input' => 'int', 'value' => false), //int or empty for false
        'encryption' => array('input' => 'select', 'value' => 'none,ssl,start_tls'), //none, ssl, start_tls
        'bind_dn' => array('input' => 'string', 'value' => false), //string or empty for false
        'bind_secret' => array('input' => 'password', 'value' => false), //string or empty for false. Could also contain a path to secret file
        'resolve_with_service_account' => array('input' => 'bool', 'value' => '1'),
        'base_dn' => array('input' => 'string', 'value' => ''),
        'user_base_dn' => array('input' => 'string', 'value' => ''),
        'user_filter' => array('input' => 'string', 'value' => ''),
        'group_base_dn' => array('input' => 'string', 'value' => ''),
        'group_filter' => array('input' => 'string', 'value' => ''),
        'groupkey' => array('input' => 'string', 'value' => ''),
        'group_recursive' => array('input' => 'bool', 'value' => '1'),
        'group_memberof' => array('input' => 'bool', 'value' => '1'),
        'upn_suffix' => array('input' => 'string', 'value' => ''),
        'userkey' => array('input' => 'string', 'value' => ''),
        'userkey_is_upn' => array('input' => 'bool', 'value' => '1'),
        'userkey_realname' => array('input' => 'string', 'value' => ''),
        'userkey_email'	=> array('input' => 'string', 'value' => ''),
        'memberkey' => array('input' => 'string', 'value' => ''),
        'memberkey_is_dn' => array('input' => 'bool', 'value' => '1'),
        'bind_with_upn' => array('input' => 'bool', 'value' => '1'),
        'ldap_options' => array('input' => 'select_string', 'value' => array('LDAP_OPT_PROTOCOL_VERSION' => 3)),
        'rename_module' => array('input' => 'string', 'value' => '')
    ),
    'Apache' => array(
        'rename_module' => array('input' => 'string', 'value' => '')
    )
);

function authdriver_disabled_in_gui($driver) {
    // we must describe the driver in $authmodule_fields in order to
    // display it in the GUI. This means that we can hide for example the
    // HeaderAuth which is a 'dangerous special case' that is missing a GUI
    // part - i.e. it's only accessible to advanced users through the shell
    global $authmodule_fields;

    if($driver == 'common') {
        return false;
    }

    return !isset($authmodule_fields[$driver]);
}

$ldap_opts = array(
    'LDAP_OPT_DEREF',
    'LDAP_OPT_DEBUG_LEVEL',
    'LDAP_OPT_ERROR_NUMBER',
    'LDAP_OPT_ERROR_STRING',
    'LDAP_OPT_HOST_NAME',
    'LDAP_OPT_MATCHED_DN',
    'LDAP_OPT_NETWORK_TIMEOUT',
    'LDAP_OPT_PROTOCOL_VERSION',
    'LDAP_OPT_REFERRALS',
    'LDAP_OPT_RESTART',
    'LDAP_OPT_SIZELIMIT',
    'LDAP_OPT_TIMELIMIT'
);

/**
 * @throws Exception if submitting a weird driver name
 * @return string
 */
function driverForm($driver, $newForm = false, $settings = false) {
    if ($newForm)
        $module = $newForm;
    else
        $module = $settings['driver'];
    global $authmodule_fields, $ldap_opts;
    $html = "<table class=''>\n";
    $html .= "<tr>\n";
    $html .= "<td><label for='$driver-driver'>Driver</label></td>\n";
    $html .= "<td><input type='text' name='$driver-driver' id='$driver-driver' value='$module' readonly='readonly' ".(authdriver_disabled_in_gui($module) ? 'disabled=disabled' : '')." /></td>\n";
    $html .= "<td> &nbsp; <img src='images/icons/shield-help.png' class='help_icon' style='cursor: pointer;' alt='driver' title='driver' /></td>\n";
    $html .= "</tr>\n";
    if(authdriver_disabled_in_gui($module)) {
        $authconf = Op5Config::instance()->getConfig('auth');
        $html .= '<tr>
                <td colspan=3><strong>This driver can not be modified through the GUI, modify /etc/op5/auth.yml manually</strong></td>
        </tr>';
        foreach($authconf[$driver] as $name => $value) {
            $html .= '<tr>';
            $html .= '<td>'.$name.'</td>';
            $html .= '<td><input type="text" disabled="disabled" value="'.$value.'" /></td>';
            $html .= '<td></td>';
            $html .= '</tr>';
        }
        $html .= '</table>';
        return $html;
    }
    $field_array = $authmodule_fields[$module];
    if (isset($settings['protocol_version'])) {
        $settings['ldap_options']['LDAP_OPT_PROTOCOL_VERSION'] = $settings['protocol_version'];
    }
    foreach ($field_array as $key => $value) {
        if ($settings) {
            @$value = $settings[$key];
        } else {
            $value = $field_array[$key]['value'];
        }
        $html .= "<tr>\n";
        if ($key === 'ldap_options') {
            $html .= "<td><label for='$driver-$key'>" . ucwords(str_replace('ldap', 'LDAP', str_replace('_', ' ', $key))) . "</label><br /><a href='http://msdn.microsoft.com/en-us/library/aa911830.aspx' target='_blank' title='Additional information can be found by following theses links. Note that all options listed on the external page might not be available through PHP or not supported by your LDAP/AD. (Opens a new window).'>Microsoft.com<span class='ui-icon ui-icon-newwin' style='display: inline-block; cursor: pointer'></span></a><br />
                <a href='http://publib.boulder.ibm.com/infocenter/iseries/v5r3/index.jsp?topic=%2Fapis%2Fldap_set_option.htm' target='_blank' title='Additional information can be found by following theses links. Note that all options listed on the external page might not be available through PHP or not supported by your LDAP/AD. (Opens a new window).'>IBM.com<span class='ui-icon ui-icon-newwin' style='display: inline-block; cursor: pointer'></span></a></td>\n";
        } else {
            $html .= "<td><label for='$driver-$key'>" . ucwords(str_replace('_', ' ', $key)) . "</label></td>\n";
        }
        $type = $field_array[$key]['input'];
        switch ($type) {
            case 'password':
                $html_nacoma_Modeles = array();
                if (is_array($value) && $key === 'bind_secret') {
                    if (isset($value['file'])) {
                        $value = 'file: ' . $value['file'];
                    } elseif (isset($value['config'])) {
                        $value = 'config: ' . $value['config'];
                    }
                }
                $class = implode(' ', $html_nacoma_Modeles);
                $checked = '';
                break;
            case 'string': case 'int':
                $html_nacoma_Modeles = array();
                if ($key === 'rename_module') {
                    $value = $driver;
                    $html_nacoma_Modeles[] = 'module_name';
                }
                $class = implode(' ', $html_nacoma_Modeles);
                $checked = '';
                break;
            case 'bool':
                $checked = '';
                if (!$newForm) {
                    $checked = ($field_array[$key]['value'] == $value) ? "checked" : "";
                }
                $value = '1';
                $class = '';
                $type = 'checkbox';
                break;
            case 'select':
                $options = explode(',', $field_array[$key]['value']);
                break;
        }

        if($type == 'select_string') {
            $html .= "<td class='ldap_options'>";
            foreach ($value as $opt => $val) {
                $html .= "<p style='width: auto;'><select name='$driver-".$key."[]' data-driver='$driver'>\n";
                $html .= "<optgroup label='SELECT LDAP OPTION AND SET VALUE'>";
                foreach ($ldap_opts as $ldap_opt) {
                    if ($opt === $ldap_opt) {
                        $html .= "<option value='$ldap_opt' selected='selected'>$ldap_opt</option>\n";
                    } else {
                        $html .= "<option value='$ldap_opt'>$ldap_opt</option>\n";
                    }
                }
                $html .= "</optgroup>";
                $html .= "</select>\n";
                $html .= "<input name='".$driver."-".$key."[]' type='text' size='6' value='$val' />\n";
                $html .= "<span class='ui-icon ui-icon-circle-minus delete_option' style='display: inline-block; cursor: pointer'></span></p>";
            }
            $html .= "<p data-driver='$driver' style='width: auto; cursor: pointer' class='add_option'><span class='ui-icon ui-icon-circle-plus' style='display: inline-block'></span>Add LDAP Option</p></td>\n";
        } else if($type == 'select') {
            $html .= "<td><select name='$driver-$key' id ='$driver-$key'>\n";
                foreach ($options as $option) {
                    $selected = "";
                    if (!$newForm)
                        $selected = ($value == $option) ? "selected='selected'" : "";
                    $html .= "<option value='$option' $selected>$option</option>\n";
                }
            $html .= "</select></td>\n";
        } else {
            $html .= "<td><input id='$driver-$key' class='$class' name='$driver-$key' type='$type' value='$value' $checked /></td>\n";
        }

        $html .= "<td> &nbsp; <img src='images/icons/shield-help.png' class='help_icon' style='cursor: pointer;' alt='$key' title='$key' />\n </td>\n";
        $html .= "</tr>\n";
    }
    $html .= "</table>";
    return $html;
}

//Handles ajax call for new driver.
if (@$_REQUEST['action'] == 'newAuthDriverForm') {
    $name = $_REQUEST['name'];
    $type = $_REQUEST['type'];
    try {
        echo driverForm($name, $type, false);
        return;
    } catch(Exception $e) {
        header('HTTP/1.1 400 Bad Request', true, 400);
        echo $e->getMessage();
        exit(1);
    }
}
elseif (@$_REQUEST['action'] == 'backupConfiguration') {
    return backupConfiguration();
}
