<?php

require_once('changelog.inc.php');

function concat_shell_args($args)
{
	$ret = '';
	foreach ($args as $a) {
		if ($ret)
			$ret .= ' ';
		$ret .= escapeshellarg($a);
	}

	return $ret;
}

function glob_execs($path)
{
	if (!is_dir($path))
		return array($path);

	$ents = glob("$path/*");
	foreach ($ents as $k => $ent) {
		if (!is_file($ent) || !is_executable($ent) || 'README' == basename($ent))
			unset($ents[$k]);
	}

	return $ents;
}

function hookify_object($obj)
{
	$str = '';
	if (empty($obj))
		return $str;

	foreach ($obj as $k => $v) {
		if (is_array($v))
			$v = join(',', $v);
		$str .= "$k=$v;";
	}

	return $str;
}

function get_full_contact_name($user)
{
	return "unknown";
}

function run_hooks($type, $objects)
{
	global $HOOKDIR;

	$log = op5Log::instance('nacoma');
	$target = "$HOOKDIR/$type";

	if (!is_executable($target))
		return;

	$user = get_user();

	$args = array();
	$nargs = 1;
	switch ($type) {
	 case 'save':
	 case 'save_denormalized':
		$args[$nargs++] = $user;
		$args[$nargs++] = get_full_contact_name($user);
		break;
	 case 'test':
		$args[$nargs++] = 'test hook';
		break;
	}

	$log->log('debug', "Running hooks for $type");

	$args = concat_shell_args($args);
	$hooks = glob_execs($target);

	$data = '';
	if (!is_array($objects)) {
		$objects = array();
		$res = nacoma_db::exec_query('SELECT * FROM changelog');
		$objs = $res->fetchAll(PDO::FETCH_ASSOC);
		foreach ($objs as $obj) {
			$objects[$obj['object_type']] = $obj;
		}
	}
	$log = op5log::instance('nacoma');
	foreach ($hooks as $hook) {
		$log->log('debug', "Running hook '$hook' for $type");
		$ph = proc_open("$hook $args", array(array('pipe', 'r'), array('pipe', 'w'), array('pipe', 'w')), $pipes);
		if(!is_resource($ph)) {
			$log->log('error', "Could not execute hook '$hook' for any objects during the '$type' event");
			continue;
		}
		$log->log('debug', "Writing changes to hook '$hook' for $type");
		foreach ($objects as $ary) {
			$ary_a = convert_changelog_object($ary['object_type'], $ary['oldstate']);
			$ary_b = convert_changelog_object($ary['object_type'], $ary['newstate']);
			if ($type == 'save_denormalized') {
				$class = $ary['object_type'] . '_nacoma_Model';
				$obj = new $class();
				$ary_a = $obj->denormalize($ary_a);
				$ary_b = $obj->denormalize($ary_b);
			}
			$pre_obj = hookify_object($ary_a);
			$post_obj = hookify_object($ary_b);
			unset($ary['oldstate']);
			unset($ary['newstate']);
			unset($ary['id']);
			$str = '';
			foreach ($ary as $k => $v)
				$str .= "$k='$v'\t";

			$str .= "pre='$pre_obj'\t";
			$str .= "post='$post_obj'";
			$str .= "\n";
			fwrite($pipes[0], $str);
		}
		fclose($pipes[0]);
		$log->log('debug', "Wrote changes to hook '$hook' for $type");
		// This assumes that as soon as STDIN is closed, the hook will
		// go to work, optionally write something to STDOUT, and then
		// exit. A misbehaving hook will lock up apache, so it would be
		// sorta cool if you'd follow that.
		stream_set_blocking($pipes[1], 0);
		stream_set_blocking($pipes[2], 0);
		$buf_len = 1024;
		// Ignore the order of stdout/stderr in favor of minimal memory
		// usage
		$log->log('debug', "Reading from output of hook '$hook' for $type");
		while(1) {
			$stdout = fread($pipes[1], $buf_len);
			if($stdout !== false) {
				echo $stdout;
			}
			$stderr = fread($pipes[2], $buf_len);
			if($stderr !== false) {
				echo $stderr;
			}
			if(feof($pipes[1]) && feof($pipes[2])) {
				break;
			}
		}
		$log->log('debug', "Read from output of hook '$hook' for $type");
		fclose($pipes[1]);
		fclose($pipes[2]);
		proc_close($ph);
		$log->log('debug', "Completed hook '$hook' for $type");
	}

	$log->log('debug', "Completed hooks for $type");
	return true;
}
