<?php

define('SKIP_KOHANA', true);
require('/opt/monitor/op5/ninja/index.php');

include_once 'include/webconfig.inc.php';
include_once 'classes/common_gui.php';
include_once 'include/nacoma_access.inc.php';
include_once 'include/data.inc.php';
require_once 'op5/auth/Auth.php';


function format_access_right( $name ) {
	return htmlentities( ucwords( str_replace( '_', ' ', $name ) ) );
}

validate_user();
$page->html_start('Configure Nagvis Access');

$access = get_nacoma_access('nagvis');
if (!in_array(NACOMA_ACCESS_ADD_DELETE, $access))
	deny_access("You need 'nagvis_add_delete' permissions to modify nagvis permissions.");

/*
 *  Which permissions to control for each map
 */
$nagvis_map_rights = array(
	'view' => 'gives the user access to the system/process information.',
	'edit' => 'gives the user access to view and change configuration.'
);

$groupclass_friendly_name = array(
	'auth_groups' => 'User group',
	'contact_groups' => 'Contact group'
);


/*
 * WRITE CHANGED CONFIG ON APPLY CLICK
*/
if(isset($_REQUEST['action'])) {

	$newperm = array();

	/* Make namemap, to map base64 tag to new group name */
	foreach($_REQUEST as $key => $value) {

		$row=explode('-',$key);
		if( count($row) != 4 ) {
			continue;
		}
		$permission = $row[0];
		$groupclass = $row[1];
		$groupname  = base64_decode( $row[2] );
		$map        = base64_decode( $row[3] );
		if( $groupname === false || $map === false ) {
			continue;
		}

		if( !isset( $nagvis_map_rights[$permission] ) ) {
			continue;
		}

		if( !empty($value ) ) {
			if(!isset($newperm[$groupclass])) $newperm[$groupclass]=array();
			if(!isset($newperm[$groupclass][$groupname])) $newperm[$groupclass][$groupname]=array();
			if(!isset($newperm[$groupclass][$groupname][$map])) $newperm[$groupclass][$groupname][$map]=array();
			if(!in_array($permission, $newperm[$groupclass][$groupname][$map])) $newperm[$groupclass][$groupname][$map][]=$permission;
		}
	}

	Op5Config::instance()->setConfig('nagvis', $newperm );
}

/*
 * END OF WRITING CHANGED CONFIG
*/

/*
 * Fetch list of maps
 * FIXME: Absolute path
 */

$nagvis_mapdir_path = '/opt/monitor/op5/nagvis_ls/etc/maps/';

$nagvis_maps = array();
foreach( glob( $nagvis_mapdir_path . '*.cfg' ) as $cfgfile ) {
	$cfgfile = basename( $cfgfile, '.cfg' );
	if( $cfgfile != '__automap' ) {
		$nagvis_maps[] = $cfgfile;
	}
}


/*
 * Fetch list of groups
 */
$op5conf = Op5Config::instance();
$nagvis_conf = $op5conf->getConfig('nagvis');

$groupclasses = array();

$auth_groups = $op5conf->getConfig('auth_groups');

/* Fetch auth groups */
$groupclasses['auth_groups'] = array_keys( $auth_groups );

/* Fetch contact groups */
$groupclasses['contact_groups'] = contactgroup_nacoma_Model::get_objects_for_user();

echo "</div><div id='object-table'>\n";

$page->form_start();
echo '<table class="ObjTable zebra" style="border-left: 0px;">';


echo '<tr class="nagvis_map_header">';
echo '<td>';
echo '</td>';
foreach( $nagvis_maps as $map ) {
	$mapnameheader = $map;
	echo '<td colspan="2" style="border-left: 1px solid #555555; padding: 3px;"><span>'.format_access_right($mapnameheader).'</span></td>';
	$mapnameheader = '';
}
echo '</tr>';

echo '<tr class="nagvis_permission_header" style="text-align: center;">';
echo '<td>';
echo '</td>';
foreach( $nagvis_maps as $map ) {
	foreach( $nagvis_map_rights as $name => $descr ) {
		$border = '';
		if ($name === 'view') {
			$border = 'border-left: 1px solid #555555;';
		}
		echo '<td style="' . $border . ' padding: 3px;"><span>'.format_access_right($name).'</span></td>';
	}
}
echo '</tr>';
//print_r( $nagvis_conf );

foreach($groupclasses as $groupclass => $groups ) {
	// Group class header
	echo '<tr>';
	echo '<td class="table_subheader" colspan="'.(1+count($nagvis_map_rights)*count($nagvis_maps)).'">';
	echo $groupclass_friendly_name[ $groupclass ];
	echo '</td>';
	echo '</tr>';

	// List of groups within class
	foreach ($groups as $groupname) {

		echo "<tr style='text-align: center;'>\n";

		echo "<td style='text-align: left;' class='group_name'>";
		echo htmlentities( $groupname, ENT_COMPAT, 'UTF-8' );
		echo "</td>\n";
		$category = "";
		foreach( $nagvis_maps as $map ) {
			foreach( $nagvis_map_rights as $name => $descr ) {
				$inputname=$name . '-' . $groupclass . '-' . base64_encode($groupname) . '-' . base64_encode($map);
				$border = '';
				if ($name === 'view') {
					$border = ' style= "border-left: 1px solid #555555;"';
				}
				echo "<td class=\"nagvis_checkbox_cell $name\" $border>";

				$checked = false;
				$disabled = false;

				/* Checked by configuration */
				if( isset( $nagvis_conf[$groupclass] ) &&
				    is_array( $nagvis_conf[$groupclass] ) &&
					isset( $nagvis_conf[$groupclass][$groupname] ) &&
					is_array( $nagvis_conf[$groupclass][$groupname] ) &&
					isset( $nagvis_conf[$groupclass][$groupname][$map] ) &&
					is_array( $nagvis_conf[$groupclass][$groupname][$map] ) &&
					in_array( $name, $nagvis_conf[$groupclass][$groupname][$map] ) ) {
						$checked = true;
						/* Drop current permission from $nagvis_conf, to enable a listing of all permissions not in table afterwards */
						$nagvis_conf[$groupclass][$groupname][$map] = array_diff( $nagvis_conf[$groupclass][$groupname][$map], array($name) );
				}

				/* Permission given globally to group by group rights, check and disable, and change name to someting not handled by above code  */
				if( $groupclass == 'auth_groups' && in_array( 'nagvis_'.$name, $auth_groups[$groupname] ) ) {
						$checked = true;
						$disabled = true;
						$inputname .= '-disabled';
				}
				echo "<input type=\"checkbox\" name=\"$inputname\"";
				if( $checked ) echo ' checked="checked"';
				if( $disabled ) echo ' disabled="disabled"';
				echo ">";
				echo "</td>\n";
			}
		}
		echo "</tr>\n\n";
	}
}

echo "</table>";

$extra_perms = array();
foreach( $nagvis_conf as $groupclass => $groups ) {
	foreach( $groups as $groupname => $maps ) {
		foreach( $maps as $mapname => $perms ) {
			foreach( $perms as $perm ) {
				$extra_perms[] = array( $groupclass, $groupname, $mapname, $perm );
			}
		}
	}
}
if( count( $extra_perms ) > 0 ) {
	echo '<div class="max magic">';
	echo '<h3>Nagvis config also contains following permissions, which will be removed when saving configuration:</h3>';
	echo '<ul>';
	foreach( $extra_perms as $extra_perm ) {
		list( $groupclass, $groupname, $mapname, $perm ) = $extra_perm;
		echo '<li>' . $groupclass_friendly_name[ $groupclass ] . ' <b>' . $groupname . '</b> can <b>' . $perm . '</b> map <b>' . $mapname . '</b></li>';
		echo '<br>';
	}
	echo '</ul>';
	echo '</div>';
}

echo '<div class="max magic">';
$page->submit_button("Save");
echo '</div>';
$page->form_end();

echo '</div>';
$page->html_end();
