<?php

/**
 * Execute a given command with asmonitor, and return the result
 */
class cmd_test_nacoma_Model
{
	/** Service status names */
	static $status = array(
		0 => 'OK',
		1 => 'WARNING',
		2 => 'CRITICAL',
		3 => 'UNKNOWN'
	);

	/** Host status names */
	static $host_status = array(
		0 => 'OK',
		1 => 'DOWN',
		2 => 'UNREACHABLE'
	);

	/**
	 * Print error and die
	 */
	function crash($msg)
	{
		echo $msg . "<br />\n";
		exit;
	}

	private $objects = array();

	/**
	 * Attach an object to the sandbox
	 */
	public function attach_object( $object ) {
		$this->objects[$object->type] = $object;
	}

	/**
	 * @param $type string 'host' etc
	 * @return array
	 * @throws Exception when object type isn't loaded or when check command is missing
	 */
	private function prepare_request($type) {
		if( !isset( $this->objects[$type] ) ) {
			throw new Exception("type $type not related to command objects\n");
		}
		$nagios_args = array();
		foreach( $this->objects as $objtype => $object ) {
			$nagios_args += $object->get_nagios_struct( $objtype .'.' );
		}
		if( $type == 'command' ) {
			$nagios_args['runcmd'] = 'test_command';
		} else {
			if( !isset($nagios_args["$type.check_command"]) ) {
				throw new Exception("no checkcommand defined");
			}
			$nagios_args['runcmd'] = $nagios_args["$type.check_command"];
		}
		
		return $nagios_args;
	}

	public function contains_dangerous_characters($result_string){
		$pattern = '/(?:\$\()+|[;&`~^\"|\'<>]/m';
		if(preg_match($pattern, $result_string)){
			return TRUE;
		};
		return FALSE;
	}

	/**
	 * Ask the running nagios instance of the command line for a given object
	 *
	 * @param $type the type of the object to ask for
	 * @param $masquerade_user if the $USERn$ vars should be masqueraded
	 * @return the command line to run
	 */
	public function cmdline($type, $masquerade_user=true)
	{
		try {
			$args = $this->prepare_request($type);
		} catch(Exception $e) {
			return "Error: ".$e->getMessage();
		}
		$qh = op5queryhandler::instance();
		if( $masquerade_user ) {
			$args = array_map(function($arg){
				return preg_replace(':\\$USER([0-9]+)\\$:','_USER$1_',$arg);
			}, $args);
		}
		$result = $qh->call('runcmd', 'expand', $args);
		
		if ($this->contains_dangerous_characters($result) === TRUE) {
			$access = get_nacoma_access("allow_dangerous_characters");
			if (!in_array(NACOMA_ACCESS_FULL_ACCESS, $access)) {
				$err_msg = "Error: illegal character encountered. Reserved characters: '$(';&`~^\"|'<>
If you need to use these characters, please enable the 'Allow Dangerous Characters' setting under Configure->Group Rights";
				throw new op5queryhandler_Exception($err_msg);
			}
		}
		return $result;
	}

	/**
	 * Run a command in a sandbox
	 *
	 * @param $type the type of the object to ask for
	 * @param $cmd_node the node on which to run the command on if different from the current/local node.
	 * @param $include_code if true, the return code will be included in the formatted output
	 * @return string the output of the plugin
	 */
	public function runcmd($type, $cmd_node=false, $include_code=true)
	{
		try {
			$args = $this->prepare_request($type);
		} catch(Exception $e) {
			return $e->getMessage();
		}

		$qh = op5queryhandler::instance();
		$result = $qh->json_call('runcmd', 'run', $args, $cmd_node);
		$output = '';

		$result_code = ($result['wait_status']>>8)&0xFF;

		switch($type) {
			case 'host':
				switch($result_code) {
					case 0: $result_code = 'UP'; break;
					case 1: $result_code = 'DOWN (check returns warning)'; break;
					case 2: $result_code = 'DOWN'; break;
					case 3: $result_code = 'UNKNOWN'; break;
				}
				break;
			case 'service':
				switch($result_code) {
					case 0: $result_code = 'OK'; break;
					case 1: $result_code = 'WARNING'; break;
					case 2: $result_code = 'CRITICAL'; break;
					case 3: $result_code = 'UNKNOWN'; break;
				}
				break;
		}

		if ($include_code)
			$output .= "Result code: " . $result_code . "\n";

		if( !empty($result['outerr']) ) {
			$output .= "ERROR OUTPUT:\n";
			$output .= $result['outerr'];
			$output .= "====================\n";
			$output .= "STANDARD OUTPUT:\n";
		}

		$output .= $result['outstd'];
		return $output;
	}
}
