<?php

/**
 * Use a host dependency when you want a host's status to depend
 * on other host(s) status(es).
 */
class hostdependency_nacoma_Model extends naemon_nacoma_Model
{
	/** object type */
	const TYPE = 'hostdependency';
	/** master object type */
	var $master_obj_type = 'host';
	var $master_obj_candidates = array('host' => 'dependent_host_name', 'hostgroup' => 'dependent_hostgroup_name');
	/** Variable in this object identifying the master object */
	var $master_obj_identifier = 'dependent_host_name';

	public $properties = array (
		'topic' => NULL,
		'var_index' => array (
			'host_name' => 'select',
			'hostgroup_name' => 'select',
			'dependent_host_name' => 'select',
			'dependent_hostgroup_name' => 'select',
			'dependency_period' => 'select',
			'execution_failure_criteria' => 'checkbox',
			'notification_failure_criteria' => 'checkbox',
			'inherits_parent' => 'bool',
			'file_id' => 'select',
			'register' => 'bool'
		),
		'widget' => array (
			'dependent_host_name' => 'hidden',
			'dependent_hostgroup_name' => 'hidden',
			'register' => 'hidden'
		),
		'default' => array (
			'execution_failure_criteria' => array (
				'n'
			),
			'notification_failure_criteria' => array (
				'd',
				'u'
			),
			'inherits_parent' => '1',
			'file_id' => 'etc/hostdependencies.cfg'
		),
		'required' => array (
			'host_name',
			'dependent_host_name',
			'notification_failure_criteria',
			'file_id'
		),
		'req_alt' => array (
			'host_name' => 'hostgroup_name',
			'dependent_host_name' => 'dependent_hostgroup_name'
		),
		'special' => array (
			'dependent_host_name' => 'host',
			'host_name' => 'host',
			'hostgroup_name' => 'hostgroup',
			'dependent_hostgroup_name' => 'hostgroup',
			'notification_failure_criteria' => array (
				'd' => 'Down',
				'u' => 'Unreachable',
				'o' => 'Up',
				'n' => false
			),
			'execution_failure_criteria' => array (
				'd' => 'Down',
				'u' => 'Unreachable',
				'o' => 'Up',
				'n' => false
			),
			'dependency_period' => 'timeperiod',
			'file_id' => 'FILE'
		)
	);

	/**
	 * Return the mayi resource for the object
	 * @see object_nacoma_Model::mayi_resource()
	 */
	public function mayi_resource() {
		return "monitor.monitoring.hostdependencies";
	}

	/**
	 * Make sure user-input is sane prior to invoking save/glue_objects
	 * @return bool: True on success. False on errors
	 */
	function validate_object()
	{
		global $DEBUG;
		/**
		 * service and dependent_service can legally be the same in 'new'
		 * if the user hasn't changed anything. This is the call for us to
		 * remove the 'new' object from the list.
		 */
		if($this->id==='new' && $this->obj[$this->master_obj_identifier] === $this->obj[substr($this->master_obj_identifier, 10)])
		{
			$this->validation_error('invalid dependency', 'Object cannot depend on itself');
			unset($this->obj);
		}

		if(!empty($this->obj) && $this->id==='new') {
			$this->obj[$this->master_obj_identifier] = $this->master_obj_id;
			$query 	= "SELECT host_name, hostgroup_name FROM " . $this->type .
			  " WHERE $this->master_obj_identifier = $this->master_obj_id";
			$result = nacoma_db::exec_query($query);
			while($row = $result->fetch(PDO::FETCH_NUM)) {
				if($row[0] == $this->obj['host_name'] && $row[1] == $this->obj['hostgroup_name']) {
					# found a duplicate dependency, so remove it
					if($DEBUG) {
						echo "Duplicate " . $this->type . ".<br />\n";
						echo "<pre>\n"; print_r($this->obj); echo "</pre>\n";
					}

					unset($this->obj);
					$this->validation_error('duplicate dependency', 'A dependency between these objects already exists');
					break;
				}
			}
			unset($result);
		}
		if (!isset($this->obj) || !$this->obj){
			return false;
		}

		return parent::validate_object();
	}

	/**
	 * Computes a master object id
	 * @return False on errors, the master object id on success
	 */
	function compute_master_obj_id()
	{
		if (!$this->master_obj_type || !$this->obj)
		  return false;

		return $this->master_obj_id;
	}

	public function get_cmd_bar_links()
	{
		$cmd_links = parent::get_cmd_bar_links();
		// instead of duplicating validation from parent, just check if the validation ended up allowing copying, and only then add clone
		if (isset($cmd_links['copy']))
			$cmd_links['clone'] = '<a title="Clone host dependencies" href="edit_special.php/copy_slaves?obj_type='.$this->type.'&amp;master_obj_id='.$this->master_obj_id.'&amp;master_obj_type='.$this->master_obj_type.'&amp;obj_id='.$this->id.'">';
		return $cmd_links;
	}

	function get_object_by_name($name)
	{
		$table = self::get_objects_for_user();
		$this->id = array_search($name, $table);
		return $this->get_object();
	}
}
