<?php

/**
 * A service runs on a host or a hostgroup. The term "service" is used very loosely. It can
 * mean an actual service that runs on the host (POP, SMTP, HTTP, etc.) or
 * some other type of metric associated with the host (response to a ping,
 * number of logged in users, free disk space, etc.).
 *
 *
 * To use it on hostgroups in the API, always specify ?parent_type=hostgroup in your calls:
 * https://monitor/api/config/service/a_hostgroup;a_service?parent_type=hostgroup
 */
class service_nacoma_Model extends naemon_nacoma_Model
{
	/** type of the object */
	const TYPE = 'service';
	/** The type of object services must live 'under' */
	var $master_obj_type = 'host';
	var $master_obj_candidates = array('host' => 'host_name', 'hostgroup' => 'hostgroup_name');
	/** variable key identifying our 'master' object */
	var $master_obj_identifier = 'host_name';
	/** The numerical value of the host which is our 'master' */
	var $master_obj_id = false;
	var $slave_types = array(
		'serviceescalation',
		'servicedependency');

	public $properties = array (
		'topic' => 'service_description',
		'var_index' => array (
			'template' => 'select',
			'host_name' => 'select',
			'hostgroup_name' => 'select',
			'service_description' => 'string',
			'display_name' => 'string',
			'is_volatile' => 'bool',
			'check_command' => 'select',
			'check_command_args' => 'string',
			'servicegroups' => 'mselect',
			'max_check_attempts' => 'int',
			'check_interval' => 'int',
			'retry_interval' => 'int',
			'active_checks_enabled' => 'bool',
			'passive_checks_enabled' => 'bool',
			'check_period' => 'select',
			'parallelize_check' => 'bool',
			'obsess' => 'bool',
			'check_freshness' => 'bool',
			'freshness_threshold' => 'int',
			'event_handler' => 'select',
			'event_handler_args' => 'string',
			'event_handler_enabled' => 'bool',
			'low_flap_threshold' => 'int',
			'high_flap_threshold' => 'int',
			'flap_detection_enabled' => 'bool',
			'flap_detection_options' => 'checkbox',
			'process_perf_data' => 'bool',
			'retain_status_information' => 'bool',
			'retain_nonstatus_information' => 'bool',
			'notification_interval' => 'int',
			'first_notification_delay' => 'int',
			'notification_period' => 'select',
			'notification_options' => 'checkbox',
			'notifications_enabled' => 'bool',
			'contacts' => 'mselect',
			'contact_groups' => 'mselect',
			'stalking_options' => 'checkbox',
			'register' => 'bool',
			'file_id' => 'select',
			'notes' => 'string',
			'notes_url' => 'notesurl',
			'action_url' => 'actionurl',
			'icon_image' => 'image',
			'icon_image_alt' => 'string'
		),
		'widget' => array (
			'host_name' => 'hidden',
			'hostgroup_name' => 'hidden',
			'register' => 'hidden'
		),
		'required' => array (
			'template',
			'host_name',
			'service_description',
			'check_command',
			'max_check_attempts',
			'check_interval',
			'retry_interval',
			'check_period',
			'notification_interval',
			'notification_period',
			'notification_options',
			'file_id'
		),
		'req_alt' => array (
			'host_name' => 'hostgroup_name'
		),
		'default' => array (
			'template' => 'default-service',
			'max_check_attempts' => '3',
			'check_interval' => '5',
			'retry_interval' => '1',
			'active_checks_enabled' => '1',
			'passive_checks_enabled' => '1',
			'check_period' => '24x7',
			'parallelize_check' => '1',
			'obsess' => '1',
			'event_handler_enabled' => '1',
			'flap_detection_enabled' => '1',
			'process_perf_data' => '1',
			'retain_status_information' => '1',
			'retain_nonstatus_information' => '1',
			'notification_period' => '24x7',
			'notification_interval' => '0',
			'notification_options' => array (
				'c',
				'w',
				'u',
				'r',
				'f'
			),
			'notifications_enabled' => '1',
			'stalking_options' => array (
				'n'
			),
			'file_id' => 'etc/services.cfg'
		),
		'visible' => array (
			'host_name',
			'hostgroup_name',
			'template',
			'service_description',
			'check_command',
			'check_command_args',
			'contact_groups',
			'file_id'
		),
		'special' => array (
			'template' => 'service_template',
			'file_id' => 'FILE',
			'notification_period' => 'timeperiod',
			'check_period' => 'timeperiod',
			'check_command' => 'command',
			'event_handler' => 'command',
			'contacts' => 'contact',
			'contact_groups' => 'contactgroup',
			'servicegroups' => 'servicegroup',
			'host_name' => 'host',
			'hostgroup_name' => 'hostgroup',
			'stalking_options' => array (
				'c' => 'Critical',
				'w' => 'Warning',
				'u' => 'Unknown',
				'o' => 'OK',
				'n' => false
			),
			'notification_options' => array (
				'c' => 'Critical',
				'w' => 'Warning',
				'u' => 'Unknown',
				'r' => 'Recovery',
				'f' => 'Flapping start and stop',
				's' => 'Scheduled downtime start and stop',
				'n' => false
			),
			'flap_detection_options' => array (
				'c' => 'Critical',
				'w' => 'Warning',
				'u' => 'Unknown',
				'o' => 'OK',
				'n' => false
			),
			'icon_image' => false /* Set in __construct */
		),
		'deprecated_names' => array (
			'obsess_over_service' => 'obsess'
		)
	);


	/**
	 * Update images specials, and call parent
	 */
	public function __construct($id = false) {
		$this->properties['special']['icon_image'] = $this->get_images();
		parent::__construct($id);
	}

	/**
	 * Return the mayi resource for the object
	 * @see object_nacoma_Model::mayi_resource()
	 */
	public function mayi_resource() {
		return "monitor.monitoring.services";
	}

	/**
	 * Weird little hint to edit.php that during object construction, we played fast and loose with the parent type
	 * Only defined for services, because that's the only type with ambiguous parents in the name.
	 */
	public $parent_autodiscovered = false;

	/**
	 * Validates an object by calling the parent function, as well
	 * as making sure no host gets two services with the same
	 * service_description assigned to it.
	 */
	function validate_object()
	{
		$result = parent::validate_object();

		$this->validate_object_identifier();

		# templates don't need the additional checks below
		if ($this->is_template)
			return $result;

		# these are marked as 'required', so the validation error list
		# will already have them listed.
		if (empty($this->obj[$this->master_obj_identifier]) || empty($this->obj['service_description']))
			return $result;

		$query = "SELECT id FROM service WHERE $this->master_obj_identifier = " .
			nacoma_db::quote_string($this->master_obj_id) . " AND service_description = " .
			nacoma_db::quote_string($this->obj['service_description']);
		if ($this->id != 'new')
			$query .= " AND id != '$this->id'";

		$res = nacoma_db::exec_query($query);
		if ($res->fetch(PDO::FETCH_NUM)) {
			$class = $this->master_obj_type . '_nacoma_Model';
			$master_obj = new $class($this->master_obj_id);

			$this->validation_error
				('service_description',
				"The $this->master_obj_type '" . $master_obj->get_object_name() . "'" .
				" already has a service named '" . $this->obj['service_description'] .
				"' associated with it");
		}
		unset($res);
		return empty($this->validation_error_list) && $result;
	}

	/**
	 * Loads a service-object based on its Nagios-name
	 *
	 * @return The loaded object, or false on failures
	 */
	function get_object_by_name($name)
	{
		if (!strstr($name, ';'))
			return parent::get_object_by_name($name);

		$tmp_parts = explode(';', $name);

		$class_name = $this->master_obj_type ."_nacoma_Model";
		$host_obj = new $class_name();
		$host_obj = $host_obj->get_object_by_name($tmp_parts[0]);
		$master_obj_id = $host_obj['id'];
		if (!$master_obj_id && $this->master_obj_type === 'host') {
			# Nobody said anything too clear about parent type, and we couldn't find a host parent?
			# It's probably a hostgroup parent.
			$this->set_master_object_type('hostgroup');
			$obj = $this->get_object_by_name($name);
			if ($obj) {
				$this->parent_autodiscovered = true;
				return $obj;
			}
			# Give up, restore master type.
			$this->set_master_object_type('host');
			return false;
		}

		$obj_name = $tmp_parts[1];
		$query = 'SELECT id FROM ' . $this->type .
			' WHERE service_description= ' .
			nacoma_db::quote_string($obj_name) . ' AND '.$this->master_obj_identifier.' =' . nacoma_db::quote_string($master_obj_id);
		$result = nacoma_db::exec_query($query);
		$row = $result->fetch(PDO::FETCH_NUM);
		unset($result);
		if (!$row) {
			# It's not a hostgroup parent, and not a host parent?
			# I bet you're mentioning the host ("monitor;PING") despite the service comming from
			# the hostgroup the host is a member of. Indirection, here I come!
			$query = 'SELECT host_hostgroup.hostgroup FROM host_hostgroup INNER JOIN host on host.id = host_hostgroup.host WHERE host.host_name = '.nacoma_db::quote_string($tmp_parts[0]);
			$res = nacoma_db::exec_query($query);
			$rows = $res->fetchAll(PDO::FETCH_NUM);
			unset($res);
			$hostgroups = array();
			# Maybe the host is a member of a hostgroup that is member of another hostgroup that provides the service. Woho!
			while (!empty($rows)) {
				$new_rows = array();
				foreach ($rows as $row) {
					$hostgroups[] = $row[0];
					$query = 'SELECT hostgroup FROM hostgroup_members WHERE hostgroup_members.members = '.nacoma_db::quote_string($row[0]);
					$res = nacoma_db::exec_query($query);
					while ($row = $res->fetch(PDO::FETCH_NUM))
						$new_rows[] = $row;
					unset($res);
				}
				$rows = $new_rows;
			}

			if (!empty($hostgroups)) {
				# What happens if there are multiple hostgroups that provide this service? I have no idea, don't do that.
				$query = 'SELECT service.id FROM service WHERE service.hostgroup_name IN ('.implode(',', $hostgroups).') AND service.service_description = '.nacoma_db::quote_string($obj_name);
				$res = nacoma_db::exec_query($query);
				$rows = $res->fetchAll(PDO::FETCH_ASSOC);
				unset($res);
				if ($rows) {
					$this->parent_autodiscovered = true;
					$this->id = $rows[0]['id'];
					return $this->get_object();
				}
			}
		} else {
			$this->id = $row[0];
			return $this->get_object();
		}
	}

	public function get_cmd_bar_links()
	{
		$cmd_links = parent::get_cmd_bar_links();
		// instead of duplicating validation from parent, just check if the validation ended up allowing copying, and only then add clone
		if (isset($cmd_links['copy']))
			$cmd_links['clone'] = '<a title="Clone services" href="edit_special.php/copy_slaves?obj_type='.$this->type.'&amp;master_obj_id='.$this->master_obj_id.'&amp;master_obj_type='.$this->master_obj_type.'&amp;obj_id='.$this->id.'">';
		return $cmd_links;
	}

	/**
	 * Given a preexisting service id, returns the type of the parent object
	 */
	public static function get_parent_type($id) {
		$query = "SELECT 1 FROM service WHERE id = ".nacoma_db::quote_string($id)." AND hostgroup_name IS NOT NULL AND hostgroup_name != ''";
		$res = nacoma_db::exec_query($query);
		$row = $res->fetch(PDO::FETCH_NUM);
		if ($row)
			$parent_type = 'hostgroup';
		else
			$parent_type = 'host';
		unset($res);
		return $parent_type;
	}

	/**
	 * Export with naming and values to map nagios struct names ond format
	 *
	 * Useful for Test this x operation
	 */
	public function get_nagios_struct($prefix='') {
		$this->expand_object();
		$obj = $this->denormalize();
				
		return array(
			$prefix.'host_name' => $obj['host_name'],
			$prefix.'description' => $obj['service_description'],
			$prefix.'display_name' => $obj['display_name']?$obj['display_name']:$obj['service_description'],
	//		$prefix.'parents' => $obj[''],
	//		$prefix.'children' => $obj[''],
			$prefix.'check_command' => $obj['check_command'] . ($obj['check_command_args']!==""?'!'.$obj['check_command_args']:''),
			$prefix.'event_handler' => $obj['event_handler'] . ($obj['event_handler_args']!==""?'!'.$obj['event_handler_args']:''),
			$prefix.'check_interval' => $obj['check_interval'],
			$prefix.'retry_interval' => $obj['retry_interval'],
			$prefix.'max_attempts' => $obj['max_check_attempts'],
			$prefix.'parallelize' => $obj['parallelize_check'],
	//		$prefix.'contact_groups' => $obj[''],
	//		$prefix.'contacts' => $obj[''],
			$prefix.'notification_interval' => $obj['notification_interval'],
			$prefix.'first_notification_delay' => $obj['first_notification_delay'],
			$prefix.'notification_options' => $this->pack_bitmask( $obj['notification_options'], array()),
			$prefix.'stalking_options' => $this->pack_bitmask( $obj['stalking_options'], array()),
		//	$prefix.'hourly_value' => $obj[''],
			$prefix.'is_volatile' => $obj['is_volatile'],
			$prefix.'notification_period' => $obj['notification_period'],
			$prefix.'check_period' => $obj['check_period'],
			$prefix.'flap_detection_enabled' => $obj['flap_detection_enabled'],
			$prefix.'low_flap_threshold' => $obj['low_flap_threshold'],
			$prefix.'high_flap_threshold' => $obj['high_flap_threshold'],
			$prefix.'flap_detection_options' => $this->pack_bitmask( $obj['flap_detection_options'], array()),
			$prefix.'process_performance_data' => $obj['process_perf_data'],
			$prefix.'check_freshness' => $obj['check_freshness'],
			$prefix.'freshness_threshold' => $obj['freshness_threshold'],
			$prefix.'accept_passive_checks' => $obj['active_checks_enabled'],
			$prefix.'event_handler_enabled' => $obj['event_handler_enabled'],
			$prefix.'checks_enabled' => $obj['active_checks_enabled'],
		//	$prefix.'check_source' => $obj[''],
			$prefix.'retain_status_information' => $obj['retain_status_information'],
			$prefix.'retain_nonstatus_information' => $obj['retain_nonstatus_information'],
			$prefix.'notifications_enabled' => $obj['notifications_enabled'],
			$prefix.'obsess' => $obj['obsess'],
			$prefix.'notes' => $obj['notes'],
			$prefix.'notes_url' => $obj['notes_url'],
			$prefix.'action_url' => $obj['action_url'],
			$prefix.'icon_image' => $obj['icon_image'],
			$prefix.'icon_image_alt' => $obj['icon_image_alt'],
	//		$prefix.'custom_variables' => $obj[''],
	/*		$prefix.'problem_has_been_acknowledged' => $obj[''],
			$prefix.'acknowledgement_type' => $obj[''],
			$prefix.'host_problem_at_last_check' => $obj[''],
			$prefix.'check_type' => $obj[''],
			$prefix.'current_state' => $obj[''],
			$prefix.'last_state' => $obj[''],
			$prefix.'last_hard_state' => $obj[''],
			$prefix.'plugin_output' => $obj[''],
			$prefix.'long_plugin_output' => $obj[''],
			$prefix.'perf_data' => $obj[''],
			$prefix.'state_type' => $obj[''],
			$prefix.'next_check' => $obj[''],
			$prefix.'should_be_scheduled' => $obj[''],
			$prefix.'last_check' => $obj[''],
			$prefix.'current_attempt' => $obj[''],
			$prefix.'current_event_id' => $obj[''],
			$prefix.'last_event_id' => $obj[''],
			$prefix.'current_problem_id' => $obj[''],
			$prefix.'last_problem_id' => $obj[''],
			$prefix.'last_notification' => $obj[''],
			$prefix.'next_notification' => $obj[''],
			$prefix.'no_more_notifications' => $obj[''],
			$prefix.'check_flapping_recovery_notification' => $obj[''],
			$prefix.'last_state_change' => $obj[''],
			$prefix.'last_hard_state_change' => $obj[''],
			$prefix.'last_time_ok' => $obj[''],
			$prefix.'last_time_warning' => $obj[''],
			$prefix.'last_time_unknown' => $obj[''],
			$prefix.'last_time_critical' => $obj[''],
			$prefix.'has_been_checked' => $obj[''],
			$prefix.'is_being_freshened' => $obj[''],
			$prefix.'notified_on' => $obj[''],
			$prefix.'current_notification_number' => $obj[''],
			$prefix.'current_notification_id' => $obj[''],
			$prefix.'latency' => $obj[''],
			$prefix.'execution_time' => $obj[''],
			$prefix.'is_executing' => $obj[''],
			$prefix.'check_options' => $obj[''],
			$prefix.'scheduled_downtime_depth' => $obj[''],
			$prefix.'pending_flex_downtime' => $obj[''],
			$prefix.'is_flapping' => $obj[''],
			$prefix.'flapping_comment_id' => $obj[''],
			$prefix.'percent_state_change' => $obj[''],
			$prefix.'modified_attributes' => $obj[''],

			$prefix.'host_ptr' => $obj[''],
			$prefix.'event_handler_ptr' => $obj[''],
			$prefix.'event_handler_args' => $obj[''],
			$prefix.'check_command_ptr' => $obj[''],
			$prefix.'check_command_args' => $obj[''],
			$prefix.'check_period_ptr' => $obj[''],
			$prefix.'notification_period_ptr' => $obj[''],
			$prefix.'servicegroups_ptr' => $obj[''],*/
			$prefix.'custom_variables' => $this->extract_custom_vars($obj)
			);
	}
}
