<?php defined('SYSPATH') OR die('No direct access allowed.');
/**
 * Configuration controller used to connect to Nacoma
 * Requires authentication
 *
 *  op5, and the op5 logo are trademarks, servicemarks, registered servicemarks
 *  or registered trademarks of op5 AB.
 *  All other trademarks, servicemarks, registered trademarks, and registered
 *  servicemarks mentioned herein may be the property of their respective owner(s).
 *  The information contained herein is provided AS IS with NO WARRANTY OF ANY
 *  KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY, AND FITNESS FOR A
 *  PARTICULAR PURPOSE.
 */
class Plugin_Controller extends Configuration_Controller {

	public function index () {

		$this->_verify_access('ninja.configuration:read');

		$search = $this->input->get('search', false);

		$this->template->css[] = 'modules/nacoma/views/css/plugin_search.css';
		$this->template->toolbar->title = "Plugins";

		Plugin_Model::load_config_commands();
		Plugin_Model::load_plugin_data();

		$plugins = array();
		$customs = array();

		foreach (Plugin_Model::$METADATA as $metadata) {
			
			$plugin = new Plugin_Model($metadata);
			$file_parts = explode('/', $metadata);
			$file_name = array_pop($file_parts);
			$metadata = str_replace('.metadata', '', $file_name);

			if ($search !== false) {
				$this->calculate_relevance($plugin, $search);
				if ($plugin->relevance >= 1)
					$plugins[] = $plugin;
			} else {
				$plugins[] = $plugin;
			}
		
		}


		foreach (Plugin_Model::$CUSTOMS as $pluginsource) {

			$plugin = new Plugin_Model();

			$path = pathinfo($pluginsource, PATHINFO_FILENAME);
			$pluginsource = ltrim($pluginsource, '.');
			$plugin->name = $path . " ( ". $pluginsource .") ";
			$output = array();

			$customs[] = $plugin;

		}
	

		$form = '<form method="get" action="' . url::method('plugin', 'index') . '">';
		$form .= '<input type="text" name="search" placeholder="Search" value="' . (($search !== false) ? addslashes($search) : "") . '" />';
		$form .= '<input type="submit" value="Search" />';
		$form .= '</form>';

		$this->template->toolbar->info($form);

		if($search !== false) {
			$plugin_count = count($plugins);
			$results = $plugin_count == 1 ? 'plugin' : 'plugins';
			$info =  "<strong>" . $plugin_count . "</strong> ";
			$info .= $results . " matching '" . html::specialchars($search) . "'";
			$this->template->toolbar->info($info);
			unset($results);
		}

		$this->template->toolbar->info('<a target="_BLANK" href="https://docs.itrsgroup.com/docs/op5-monitor/current/topics/admin-manual/plugins/plugins.html">More about Plugins</a>');


		$this->template->content = new View('plugin_search');
		$this->template->content->search_term = $search;
		$this->template->content->plugins = $plugins;
		$this->template->content->customs = $customs;

	}

	private function relevance ($term, $subject, $weight) {

		if (!isset($subject)) return 0;
		if (is_array($subject)) {

			$relevance = 0;

			foreach ($subject as $item) {
				$relevance += $this->relevance($term, $item, $weight);
			}

			return $relevance;

		}

		$full_match = 0; 
		if (strtolower($term) == strtolower($subject))
			$full_match = 100;

		$terms = array(); 
		foreach(explode( " ", $term ) as $t) {
			$terms[] = preg_quote($t, '/');
		}

		$terms = "/(" . implode("|", $terms) . ")/i";
		$match = preg_match_all($terms, $subject, $matches);

		$similarity = 0;

		if (is_array($terms)) { 
			foreach ($terms as $t) {
				$similarity += similar_text($subject, $t);
			}
		}

		$rel = ($match * $weight);
		return round($rel + ($similarity / 50)) + $full_match;

	}

	private function calculate_relevance ($plugin, $term) {
		
		$plugin->relevance += $this->relevance($term, $plugin->name, 8);
		$plugin->relevance += $this->relevance($term, $plugin->categories, 4);
		$plugin->relevance += $this->relevance($term, $plugin->tags, 6);
		$plugin->relevance += $this->relevance($term, $plugin->info, 2);
		$plugin->relevance += $this->relevance($term, $plugin->why, 1);
		$plugin->relevance += $this->relevance($term, $plugin->how, 3);

		foreach ($plugin->commands as $name => $command) {

			$cterm = preg_replace("/check\_/i", "", $term);
			$plugin->relevance += $this->relevance($cterm, $name, 4);

			if (isset($command["tag"])) {
				$plugin->relevance += $this->relevance($term, $command["tag"], 6);
			}

		}
	}

}
