<?php

/**
 * An exception thrown when there is a validation error
 */
class widget_validation_exception extends Exception {}

/**
 * A widget really only does two things: it can print itself,
 * and parse the resulting input data (and validate it, if applicable)
 */
abstract class Base_Nacomawidget {

	/**
	 * Return a nice html id string for the widget
	 */
	public function get_id($obj, $token) {
		return "$obj->type[$obj->id][$token]";
	}

	/**
	 * Return the label tag
	 */
	public function get_label($obj, $token, $required = false) {
		$token_label = isset($obj->properties['label'][$token]) ? $obj->properties['label'][$token] : $token;
		$extra = null;
		if($required) {
			$extra = " class='required' ";
		}
		return "<label $extra for=\"".html::specialchars($this->get_id($obj, $token))."\">$token_label</label>";
	}

	/**
	 * Print a complete row in a widget table
	 */
	public function render_row($obj, $token, $value) {
		global $page;

		echo "<tr class=\"" . str_replace(' ', '_', $token) . "_row\">\n \n";
		echo " <td class=\"HelpList\"><a href=\"#\" title=\"Click to view help on ".$token."\" tabindex='-1' data-help='".$token."'><img src=\"images/icons/shield-help.png\" alt=\"Help\" title=\"Help\" /></a></td>";
		echo "<td>";
		# print the checkbox used to select variables for
		# propagation, but we don't do it for commands or
		# variables used to uniquely identify an object
		if ($obj->type !== 'command' && $token != $obj->type . '_name' && !empty($obj->id)) {
			echo $page->print_propagate_checkbox($token);
		}
		$required = false;
		if(is_array($obj->properties['required']) && in_array($token, $obj->properties['required'])) {
			$required = true;
		}
		echo "  </td>\n";
		echo " <td class=\"VarList " . str_replace(' ', '_', $token) . "_namefield\">".$this->get_label($obj, $token, $required)."</td>\n";

		echo "  <td class=\"" . str_replace(' ', '_', $token) . "_valuefield\">";

		$this->render($obj, $token, $value);

		echo "  </td>";
		echo "  \n</tr>";
	}

	/**
	 * Print widget
	 */
	abstract function render($obj, $token, $value);

	/**
	 * Validate and return widget's value
	 * @throws widget_validation_exception
	 */
	static function validate($obj, $token, $value) {
		return $value;
	}
}
