<?php

/**
 * Multi-select input widget
 */
class Mselect_Nacomawidget extends Select_Nacomawidget {
	/**
	 * Creates 2 multiple select boxes with unselected alternatives
	 * on the left and selected on the right.
	 * @param $tokenvalues array: Alternatives to list, in key => value form
	 * @param $dflt The default-selected items
	 * @param $select_name html form name
	 */
	public static function make_multi_selection( $type, $dflt, $select_name = 'selection', $exclude = false)
	{
		global $page;

		$select_id = html::specialchars( $select_name ) . "[]";

		if(!$dflt) {
			$dflt = array();
		}
		if (!is_array($type)) {
			$default = "";
			if ($dflt) {
				foreach ( $dflt as $name ) {
					$default .= '<option value="' . htmlspecialchars($name) . '">' . htmlspecialchars($name) . '</option>';
				}
			}
			if (!$exclude)
				$exclude = array();
			else
				$exclude = (array)$exclude;

			printf('<select id="%1$s" name="%1$s" data-type="%2$s" multiple="multiple" data-filterable="true" %3$s>%4$s</select>', $select_id, $type, $exclude ? 'data-exclude="'.htmlspecialchars(json_encode($exclude)).'"' : '', $default);
		} else {
			printf('<select id="%1$s" name="%1$s" multiple="multiple" data-filterable="true" %2s>', $select_id, $exclude ? 'data-exclude="'.htmlspecialchars(json_encode($exclude)).'"' : '');
			foreach ($type as $v) {
				printf('<option value="%s"%s>%s</option>', htmlspecialchars($v), in_array($v, $dflt) ? ' selected="selected"' : '', htmlspecialchars($v));
			}
			printf('</select>');
		}
		if (is_array($exclude)) {
			foreach ($exclude as $id) {
				$page->hidden_var('select_exclude['.$type.'][]', $id);
			}
		}

		return true;

	}

	function render( $obj, $token, $value ) {
		$type = $obj->properties[ 'special' ][ $token ];
		$exclude = array();
		if($obj->id > 1 && $type == $obj->type) {
			$exclude = $obj->get_object_name();
		}
		self::make_multi_selection($type, $value, $this->get_id( $obj, $token ), $exclude);
	}

	public function get_label($obj, $token, $required = false) {
		$token_label = isset($obj->properties['label'][$token]) ? $obj->properties['label'][$token] : $token;
		$extra = null;
		if($required) {
			$extra = " class='required' ";
		}

		return "<label $extra for=\"".html::specialchars($this->get_id($obj, $token))."[]\">$token_label</label>";
	}

	static function validate($obj, $token, $value) {
		if (!$value)
			return array();
		$spec = $obj->properties['special'];
		if (is_array($spec[$token])) {
			foreach ($value as $k => $v) {
				if (!isset($spec[$token][$v]))
					unset($value[$k]);
			}
			return $value;
		}
		// templates can link to their specific grouptypes,
		// so we need to check for that too
		$class = $spec[$token] . '_nacoma_Model';
		$tmplclass = $spec[$token] . '_template_nacoma_Model';
		foreach ($value as $id) {
			if (!$class::object_exists($id) &&
				!$tmplclass::object_exists($id))
			{
				throw new widget_validation_exception("$token points to {$spec[$token]} type objects, but '$id' doesn't exist");
			}
		}
		return $value;
	}
}
