#!/usr/bin/env python
#
# lib for writing nacoma hooks

class Change(object):
    '''Parses and represents a change as retrieved from the nacoma hook API'''
    __slots__ = ['type', 'id', 'oldname', 'newname', 'username', 'time', 'pre', 'post']

    def __init__(self, line):
        parts = line.split('\t')
        if len(parts) == 1:
            raise Exception("Invalid change line, not tab separated: %s" % line)
        for part in parts:
            (key, val) = part.split('=', 1)
            val = val.strip("' \n")
            if key == 'object_type':
                self.type = val
            elif key == 'object_id':
                self.id = val
            elif key == 'oldname':
                self.oldname = val
            elif key == 'newname':
                self.newname = val
            elif key == 'username':
                self.username = val
            elif key == 'time':
                self.time = val
            elif key == 'pre':
                self.pre = val
            elif key == 'post':
                self.post = val

    def is_new(self):
        return (not self.oldname and self.newname) or (not self.pre and self.post)
    def is_deleted(self):
        return (self.oldname and not self.newname) or (self.pre and not self.post)
    def is_renamed(self):
        return self.newname and self.oldname and self.pre and self.post and self.newname != self.oldname
